@@ -83,9 +83,12 @@ def parse_config_list(f, outfmt, out):
         parts = line.strip().split()
         config=parts[0]
         results=None
-        if len(parts) == 2:
+        description=None
+        if len(parts) >= 2:
             results=parts[1]
-        report(config, outfmt, out, results)
+        if len(parts) >= 3:
+            description=parts[2]
+        report(config, outfmt, out, results, description)
 
 
 def get_path_to_executable(exe):
@@ -97,7 +100,7 @@ def get_path_to_executable(exe):
     return retval
 
 
-def report(cfg, outfmt, out, results=None):
+def report(cfg, outfmt, out, results=None, description=None):
     elb = get_path_to_executable("elastic-blast")
     elb_cost = get_path_to_executable("elb-cost")
 
@@ -114,7 +117,7 @@ def report(cfg, outfmt, out, results=None):
     csp = o['clusterInfo']['provider']
     cmd = [ elb_cost ]
     if csp == 'AWS':
-        if 'name' in o['clusterInfo']:
+        if 'name' in o['clusterInfo'] and len(o['clusterInfo']['name']) > 0:
             cluster_name = o['clusterInfo']['name']
             cmd.append('--aws-cluster-name')
             cmd.append(cluster_name)
@@ -133,15 +136,26 @@ def report(cfg, outfmt, out, results=None):
     mo = re.search(r'[$](\d+.\d*)', res)
     if mo:
         cost = '$' + str(mo.group(1))
-    if outfmt == 'txt':
-        print(f'Cluster {cluster_name}, time {wall_clock}, cost {cost}, provider {csp}', file=out)
-    elif outfmt in ('csv', 'tsv', 'jira'):
-        separator = {'csv': ',', 'tsv': '\t', 'jira': '|'}[outfmt]
-        if outfmt == 'jira':
-            fields = ['', cluster_name, wall_clock, cost, csp, '']
-        else:
-            fields = [cluster_name, wall_clock, cost, csp]
-        print(separator.join(fields), file=out)
+    if description is not None:
+        if outfmt == 'txt':
+            print(f'Cluster {cluster_name}, Description {description}, time {wall_clock}, cost {cost}, provider {csp}', file=out)
+        elif outfmt in ('csv', 'tsv', 'jira'):
+            separator = {'csv': ',', 'tsv': '\t', 'jira': '|'}[outfmt]
+            if outfmt == 'jira':
+                fields = ['', cluster_name, description, wall_clock, cost, csp, '']
+            else:
+                fields = [cluster_name, description, wall_clock, cost, csp]
+            print(separator.join(fields), file=out)
+    else:
+        if outfmt == 'txt':
+            print(f'Cluster {cluster_name}, time {wall_clock}, cost {cost}, provider {csp}', file=out)
+        elif outfmt in ('csv', 'tsv', 'jira'):
+            separator = {'csv': ',', 'tsv': '\t', 'jira': '|'}[outfmt]
+            if outfmt == 'jira':
+                fields = ['', cluster_name, wall_clock, cost, csp, '']
+            else:
+                fields = [cluster_name, wall_clock, cost, csp]
+            print(separator.join(fields), file=out)
 
 
 def main():
